const PROD_SEARCH_FIELDS = ['name', 'fullname', 'externalCode'];

const PROCESSOR_COLLECTION = 'productProccessors';
const FAMILY_COLLECTION = 'ProductFamilies';
const PROD_COLLECTION = 'Products';
const SUBPROD_COLLECTION = 'Subproducts';
const PRODENABLE_COLLECTION = 'EnabledProducts';

const moduleErrors = {
  createProductProccessor: {
    alreadyExist: ['01', 'El procesador de productos ya existe'],
  },
  findProcessor: {
    notFound: ['01', 'El procesador de productos no existe'],
  },
  modifyProcessor: {
    alreadyActive: ['01', 'El procesador de productos ya está activo'],
    alreadyInactive: ['02', 'El procesador de productos no está activo'],
  },
  createProductFamily: {
    alreadyExist: ['01', 'La familia de productos ya existe'],
  },
  findFamily: {
    notFound: ['01', 'La familia de productos no existe'],
    hasChildren: ['02', 'Familia de productos contiene subproductos'],
  },
  modifyFamily: {
    alreadyActive: ['01', 'La familia de productos ya está activo'],
    alreadyInactive: ['02', 'La familia de productos no está activo'],
  },
  createProduct: {
    alreadyExist: ['01', 'Producto ya existe'],
  },
  findProduct: {
    notFound: ['01', 'Producto no existe'],
    hasChildren: ['02', 'Producto contiene subproductos'],
  },
  modifyProduct: {
    alreadyActive: ['01', 'Producto ya está activo'],
    alreadyInactive: ['02', 'Producto no está activo'],
  },
  createSubProduct: {
    alreadyExist: ['01', 'Subproducto ya existe'],
  },
  findSubProduct: {
    notFound: ['01', 'Subproducto no existe'],
  },
  modifySubProduct: {
    alreadyActive: ['01', 'Subproducto ya está activo'],
    alreadyInactive: ['02', 'Subproducto no está activo'],
  },
  createEnableProduct: {
    alreadyExist: ['01', 'Habilitación de producto ya existe'],
  },
  findEnableProduct: {
    notFound: ['01', 'Habilitación de producto no existe'],
    alreadyExist: ['02', 'Habilitación de producto ya existe'],
  },
  modifyEnableProduct: {
    alreadyActive: ['01', 'Habilitación de producto ya está activo'],
    alreadyInactive: ['02', 'Habilitación de producto no está activo'],
  },
};


class ProductManager {

  constructor(getDb, logger, errMgr, productQueries) {
    this.getDb = getDb;
    this.logger = logger;
    this.errMgr = errMgr;
    this.productQueries = productQueries;
    this.errMgr.addModuleSet('products', moduleErrors);

  }

  async getProcessors(reqData, constraints = null) {
    const procCol = this.getDb().collection(PROCESSOR_COLLECTION);
    const procList = await procCol.find(this.productQueries.findByAttributes(constraints, PROD_SEARCH_FIELDS));
    const processors = (procList) ? await procList.toArray() : null;
    if (!processors || processors.length < 1) {
      const errorObj = this.errMgr.get(moduleErrors.findProcessor.notFound);
      return [errorObj, null];
    }
    return [null, { processors }];
  }

  async createProductProccessor(processorData) {
    const { name } = processorData;
    const procCol = this.getDb().collection(PROCESSOR_COLLECTION);
    const procDetail = await procCol.findOne(this.productQueries.findByName(name));
    if (procDetail) {
      const errorDetail = `Procesador de productos ${name} ya existe`;
      const errorObj = this.errMgr.get(moduleErrors.createProductProccessor.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newProcData = { ...processorData, enabled: false, creation: new Date() };
    const actionResult = await procCol.insertOne(newProcData);
    this.logger.debug(`Creación de un nuevo procesador ${newProcData} con resultado ${actionResult?.insertedId}`);
    return [null, { envId: actionResult?.insertedId }];
  }

  async updateProductProccessor(reqData) {
    const { productProcesorId } = reqData;
    const procCol = this.getDb().collection(PROCESSOR_COLLECTION);
    const procDetail = await procCol.findOne(this.productQueries.findById(productProcesorId));
    if (!procDetail) {
      const errorDetail = `Procesador con Id ${productProcesorId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProcessor.notFound, errorDetail);
      return [errorObj, null];
    }
    const updateData = { ...reqData };
    delete updateData.productProcesorId;
    const actionResult = await procCol.updateOne(this.productQueries.findById(productProcesorId),
      this.productQueries.updateObj(updateData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Modificación del procesador ${updateData} con resultado ${result}`);
    return [null, { result }];
  }

  async activateProductProccessor(reqData) {
    const { productProcesorId } = reqData;
    const procCol = this.getDb().collection(PROCESSOR_COLLECTION);
    const procDetail = await procCol.findOne(this.productQueries.findById(productProcesorId));
    if (!procDetail) {
      const errorDetail = `Procesador con Id ${productProcesorId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProcessor.notFound, errorDetail);
      return [errorObj, null];
    }
    if (procDetail.enabled) {
      const errorDetail = `Procesador con Id ${productProcesorId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyProcessor.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await procCol.updateOne(this.productQueries.findById(productProcesorId),
      this.productQueries.updateObj({ enabled: true }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación del procesador ${productProcesorId} con resultado ${result}`);
    return [null, { result }];
  }

  async inactivateProductProccessor(reqData) {
    const { productProcesorId } = reqData;
    const procCol = this.getDb().collection(PROCESSOR_COLLECTION);
    const procDetail = await procCol.findOne(this.productQueries.findById(productProcesorId));
    if (!procDetail) {
      const errorDetail = `Procesador con Id ${productProcesorId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProcessor.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!procDetail.enabled) {
      const errorDetail = `Relación con Id ${productProcesorId} actualmente inactivo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyProcessor.alreadyInactive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await procCol.updateOne(this.productQueries.findById(productProcesorId),
      this.productQueries.updateObj({ enabled: false }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Inactivación del procesador ${productProcesorId} con resultado ${result}`);
    return [null, { result }];
  }

  /**
   * Familias de productos
   */

  async createProductFamily(familyData) {
    const { name } = familyData;
    const familyCol = this.getDb().collection(FAMILY_COLLECTION);
    const familyDetail = await familyCol.findOne(this.productQueries.findByName(name));
    if (familyDetail) {
      const errorDetail = `Familia de productos ${name} ya existe`;
      const errorObj = this.errMgr.get(moduleErrors.createProductFamily.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newFamilyData = { ...familyData, enabled: false, creation: new Date() };
    const actionResult = await familyCol.insertOne(newFamilyData);
    this.logger.debug(`Creación de una nueva familia de productos ${newFamilyData} con resultado ${actionResult?.insertedId}`);
    return [null, { productFamilyId: actionResult?.insertedId }];
  }

  async updateProductFamily(reqData) {
    const { productFamilyId } = reqData;
    const familyCol = this.getDb().collection(FAMILY_COLLECTION);
    const familyDetail = await familyCol.findOne(this.productQueries.findById(productFamilyId));
    if (!familyDetail) {
      const errorDetail = `Familia de productos con Id ${productFamilyId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findFamily.notFound, errorDetail);
      return [errorObj, null];
    }
    const updateData = { ...reqData };
    delete updateData.productFamilyId;
    const actionResult = await familyCol.updateOne(this.productQueries.findById(productFamilyId),
      this.productQueries.updateObj(updateData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Modificación de la familia ${updateData} con resultado ${result}`);
    return [null, { result }];
  }

  async activateProductFamily(reqData) {
    const { productFamilyId } = reqData;
    const familyCol = this.getDb().collection(FAMILY_COLLECTION);
    const familyDetail = await familyCol.findOne(this.productQueries.findById(productFamilyId));
    if (!familyDetail) {
      const errorDetail = `Familia de produtos con Id ${productFamilyId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findFamily.notFound, errorDetail);
      return [errorObj, null];
    }
    if (familyDetail.enabled) {
      const errorDetail = `Familia de produtos con Id ${productFamilyId} actualmente activa`;
      const errorObj = this.errMgr.get(moduleErrors.modifyFamily.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await familyCol.updateOne(this.productQueries.findById(productFamilyId),
      this.productQueries.updateObj({ enabled: true }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación de la familia ${productFamilyId} con resultado ${result}`);
    return [null, { result }];
  }

  async inactivateProductFamily(reqData) {
    const { productFamilyId } = reqData;
    const familyCol = this.getDb().collection(FAMILY_COLLECTION);
    const familyDetail = await familyCol.findOne(this.productQueries.findById(productFamilyId));
    if (!familyDetail) {
      const errorDetail = `Familia de produtos con Id ${productFamilyId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findFamily.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!familyDetail.enabled) {
      const errorDetail = `Familia de produtos con Id ${productFamilyId} actualmente inactiva`;
      const errorObj = this.errMgr.get(moduleErrors.modifyFamily.alreadyInactive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await familyCol.updateOne(this.productQueries.findById(productFamilyId),
      this.productQueries.updateObj({ enabled: false }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Inactivación de la familia ${productFamilyId} con resultado ${result}`);
    return [null, { result }];
  }

  async deleteProductFamily(reqData) {
    const { productFamilyId } = reqData;
    const familyCol = this.getDb().collection(FAMILY_COLLECTION);
    const familyDetail = await familyCol.findOne(this.productQueries.findById(productFamilyId));
    if (!familyDetail) {
      const errorDetail = `Familia de produtos con Id ${productFamilyId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findFamily.notFound, errorDetail);
      return [errorObj, null];
    }
    if (familyDetail.enabled) {
      const errorDetail = `Familia de produtos con Id ${productFamilyId} actualmente activa`;
      const errorObj = this.errMgr.get(moduleErrors.modifyFamily.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const productCol = this.getDb().collection(PROD_COLLECTION);
    const productCount = await productCol.find(this.productQueries.findProductByFamily(productFamilyId)).count();
    if (productCount > 0) {
      const errorDetail = `Familia de produtos con Id ${productFamilyId} contiene productos`;
      const errorObj = this.errMgr.get(moduleErrors.findFamily.hasChildren, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await familyCol.deleteOne(this.productQueries.findById(productFamilyId));
    this.logger.debug(`Eliminación de la familia ${productFamilyId} con resultado ${actionResult?.result?.n}`);
    const result = actionResult?.result?.n > 0;
    return [null, { result }];
  }

  /**
   * Productos
   */

  async createProduct(productData) {
    const { productFamilyId, proccessorId, name } = productData;
    const productCol = this.getDb().collection(PROD_COLLECTION);
    const productDetailPrms = productCol.findOne(this.productQueries.findByName(name));
    const familyCol = this.getDb().collection(FAMILY_COLLECTION);
    const familyDetailPrms = familyCol.findOne(this.productQueries.findById(productFamilyId));
    const procCol = this.getDb().collection(PROCESSOR_COLLECTION);
    const procDetailPrms = procCol.findOne(this.productQueries.findById(proccessorId));
    const [productDetail, familyDetail, procDetail] = await Promise
      .all([productDetailPrms, familyDetailPrms, procDetailPrms]);
    if (productDetail) {
      const errorDetail = `Producto ${name} ya existe`;
      const errorObj = this.errMgr.get(moduleErrors.createProduct.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    if (!familyDetail) {
      const errorDetail = `Familia de productos ${name} no existe`;
      const errorObj = this.errMgr.get(moduleErrors.findFamily.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!procDetail) {
      const errorDetail = `Procesador de productos ${name} no existe`;
      const errorObj = this.errMgr.get(moduleErrors.findProcessor.notFound, errorDetail);
      return [errorObj, null];
    }
    const newProdData = { ...productData, enabled: false, creation: new Date() };
    const actionResult = await productCol.insertOne(newProdData);
    this.logger.debug(`Creación de un nuevo producto ${newProdData} con resultado ${actionResult?.insertedId}`);
    return [null, { productId: actionResult?.insertedId }];
  }

  async updateProduct(reqData) {
    const { productId, name, productFamilyId, proccessorId } = reqData;
    const productCol = this.getDb().collection(PROD_COLLECTION);
    const productDetail = await productCol.findOne(this.productQueries.findById(productId));
    if (!productDetail) {
      const errorDetail = `Producto con Id ${productId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    if (name) {
      const productCol = this.getDb().collection(PROD_COLLECTION);
      const productWithName = await productCol.findOne(this.productQueries.findByName(name));
      if (productWithName) {
        const errorDetail = `Producto ${name} ya existe`;
        const errorObj = this.errMgr.get(moduleErrors.createProduct.alreadyExist, errorDetail);
        return [errorObj, null];
      }
    }
    if (productFamilyId) {
      const familyCol = this.getDb().collection(FAMILY_COLLECTION);
      const familyDetail = await familyCol.findOne(this.productQueries.findById(productFamilyId));
      if (!familyDetail) {
        const errorDetail = `Familia de productos ${name} no existe`;
        const errorObj = this.errMgr.get(moduleErrors.findFamily.notFound, errorDetail);
        return [errorObj, null];
      }
    }
    if (proccessorId) {
      const procCol = this.getDb().collection(PROCESSOR_COLLECTION);
      const procDetail = await procCol.findOne(this.productQueries.findById(proccessorId));
      if (!procDetail) {
        const errorDetail = `Procesador de productos ${name} no existe`;
        const errorObj = this.errMgr.get(moduleErrors.findProcessor.notFound, errorDetail);
        return [errorObj, null];
      }
    }
    const updateData = { ...reqData };
    delete updateData.productId;
    const actionResult = await productCol.updateOne(this.productQueries.findById(productId),
      this.productQueries.updateObj(updateData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Modificación del producto ${updateData} con resultado ${result}`);
    return [null, { result }];
  }

  async activateProduct(reqData) {
    const { productId } = reqData;
    const productCol = this.getDb().collection(PROD_COLLECTION);
    const productDetail = await productCol.findOne(this.productQueries.findById(productId));
    if (!productDetail) {
      const errorDetail = `Producto con Id ${productId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    if (productDetail.enabled) {
      const errorDetail = `Producto con Id ${productId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyProduct.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await productCol.updateOne(this.productQueries.findById(productId),
      this.productQueries.updateObj({ enabled: true }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación del producto ${productId} con resultado ${result}`);
    return [null, { result }];
  }

  async inactivateProduct(reqData) {
    const { productId } = reqData;
    const productCol = this.getDb().collection(PROD_COLLECTION);
    const productDetail = await productCol.findOne(this.productQueries.findById(productId));
    if (!productDetail) {
      const errorDetail = `Producto con Id ${productId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!productDetail.enabled) {
      const errorDetail = `Producto con Id ${productId} actualmente inactivo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyProduct.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await productCol.updateOne(this.productQueries.findById(productId),
      this.productQueries.updateObj({ enabled: false }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Inactivación del producto ${productId} con resultado ${result}`);
    return [null, { result }];
  }

  async deleteProduct(reqData) {
    const { productId } = reqData;
    const productCol = this.getDb().collection(PROD_COLLECTION);
    const productDetail = await productCol.findOne(this.productQueries.findById(productId));
    if (!productDetail) {
      const errorDetail = `Producto con Id ${productId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    if (productDetail.enabled) {
      const errorDetail = `Producto con Id ${productId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyProduct.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const subprodCol = this.getDb().collection(SUBPROD_COLLECTION);
    const subprodCount = await subprodCol.find(this.productQueries.findSubProdByProd(productId)).count();
    if (subprodCount > 0) {
      const errorDetail = `Produto con Id ${productId} contiene sub-productos`;
      const errorObj = this.errMgr.get(moduleErrors.findProduct.hasChildren, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await productCol.deleteOne(this.productQueries.findById(productId));
    this.logger.debug(`Eliminación del producto ${productId} con resultado ${actionResult?.result?.n}`);
    const result = actionResult?.result?.n > 0;
    return [null, { result }];
  }

  /**
   * Sub-Productos
   */

  async createSubproduct(subProdData) {
    const { productId, name } = subProdData;
    const subprodCol = this.getDb().collection(SUBPROD_COLLECTION);
    const subprodDetail = await subprodCol.findOne(this.productQueries.findByName(name));
    if (subprodDetail) {
      const errorDetail = `Subproducto ${name} ya existe`;
      const errorObj = this.errMgr.get(moduleErrors.createSubProduct.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const productCol = this.getDb().collection(PROD_COLLECTION);
    const productDetail = await productCol.findOne(this.productQueries.findById(productId));
    if (!productDetail) {
      const errorDetail = `Producto con Id ${productId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    const newSubProdData = { ...subProdData, enabled: false, creation: new Date() };
    const actionResult = await subprodCol.insertOne(newSubProdData);
    this.logger.debug(`Creación de un nuevo subproducto ${newSubProdData} con resultado ${actionResult?.insertedId}`);
    return [null, { subproductId: actionResult?.insertedId }];
  }

  async updateSubproduct(reqData) {
    const { subproductId, productId } = reqData;
    const subprodCol = this.getDb().collection(SUBPROD_COLLECTION);
    const subprodDetail = await subprodCol.findOne(this.productQueries.findById(subproductId));
    if (!subprodDetail) {
      const errorDetail = `Subproducto con Id ${subproductId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findSubProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    if (productId) {
      const productCol = this.getDb().collection(PROD_COLLECTION);
      const productDetail = await productCol.findOne(this.productQueries.findById(productId));
      if (!productDetail) {
        const errorDetail = `Producto con Id ${productId} no encontrado`;
        const errorObj = this.errMgr.get(moduleErrors.findProduct.notFound, errorDetail);
        return [errorObj, null];
      }
    }
    const updateData = { ...reqData };
    delete updateData.subproductId;
    const actionResult = await subprodCol.updateOne(this.productQueries.findById(subproductId),
      this.productQueries.updateObj(updateData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Modificación del subproducto ${updateData} con resultado ${result}`);
    return [null, { result }];
  }

  async activateSubproduct(reqData) {
    const { subproductId } = reqData;
    const subprodCol = this.getDb().collection(SUBPROD_COLLECTION);
    const subprodDetail = await subprodCol.findOne(this.productQueries.findById(subproductId));
    if (!subprodDetail) {
      const errorDetail = `Subproducto con Id ${subproductId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findSubProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    if (subprodDetail.enabled) {
      const errorDetail = `Subproducto con Id ${subproductId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifySubProduct.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await subprodCol.updateOne(this.productQueries.findById(subproductId),
      this.productQueries.updateObj({ enabled: true }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación del subproducto ${subproductId} con resultado ${result}`);
    return [null, { result }];
  }

  async inactivateSubproduct(reqData) {
    const { subproductId } = reqData;
    const subprodCol = this.getDb().collection(SUBPROD_COLLECTION);
    const subprodDetail = await subprodCol.findOne(this.productQueries.findById(subproductId));
    if (!subprodDetail) {
      const errorDetail = `Subproducto con Id ${subproductId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findSubProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!subprodDetail.enabled) {
      const errorDetail = `Subproducto con Id ${subproductId} actualmente inactivo`;
      const errorObj = this.errMgr.get(moduleErrors.modifySubProduct.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await subprodCol.updateOne(this.productQueries.findById(subproductId),
      this.productQueries.updateObj({ enabled: false }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Inactivación del subproducto ${subproductId} con resultado ${result}`);
    return [null, { result }];
  }

  async deleteSubproduct(reqData) {
    const { subproductId } = reqData;
    const subprodCol = this.getDb().collection(SUBPROD_COLLECTION);
    const subprodDetail = await subprodCol.findOne(this.productQueries.findById(subproductId));
    if (!subprodDetail) {
      const errorDetail = `Subproducto con Id ${subproductId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findSubProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    if (subprodDetail.enabled) {
      const errorDetail = `Subproducto con Id ${subproductId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifySubProduct.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await subprodCol.deleteOne(this.productQueries.findById(subproductId));
    const result = actionResult?.result?.n > 0;
    this.logger.debug(`Eliminación del subproducto ${subproductId} con resultado ${result}`);
    return [null, { result }];
  }

  /**
   * Asociación de producto a grupo de terceros
   */

  async enableGroupProduct(reqData) {
    const { productId, subproductId, groupId } = reqData;
    const enableProdCol = this.getDb().collection(PRODENABLE_COLLECTION);
    const enableProdDetail = await enableProdCol.findOne(this.productQueries.findEnableProd(productId, subproductId, groupId));
    if (enableProdDetail) {
      const errorDetail = `Habilitación de producto ya existente`;
      const errorObj = this.errMgr.get(moduleErrors.findEnableProduct.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newGroupProduct = { ...reqData, creation: new Date() };
    const actionResult = await enableProdCol.insertOne(newGroupProduct);
    this.logger.debug(`Habilitación de producto en grupo ${newGroupProduct} con resultado ${actionResult?.insertedId}`);
    return [null, { enabledProductId: actionResult?.insertedId }];
  }

  async disableGroupProduct(reqData) {
    const { enabledProductId } = reqData;
    const enableProdCol = this.getDb().collection(PRODENABLE_COLLECTION);
    const enableProdDetail = await enableProdCol.findOne(this.productQueries.findById(enabledProductId));
    if (!enableProdDetail) {
      const errorDetail = `Habilitación de producto ${enabledProductId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findEnableProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await enableProdCol.deleteOne(this.productQueries.findById(enabledProductId));
    const result = actionResult?.result?.n > 0;
    this.logger.debug(`Eliminación del producto de grupo ${enabledProductId} con resultado ${result}`);
    return [null, { result }];
  }

  /**
   * Asociación directa de producto a un tercero
   */

  async enableThirdPartyProduct(reqData) {
    const { productId, subproductId, thirdPartyId } = reqData;
    const enableProdCol = this.getDb().collection(PRODENABLE_COLLECTION);
    const enableProdDetail = await enableProdCol.findOne(this.productQueries.findEnableProd(productId, subproductId, null, thirdPartyId));
    if (enableProdDetail) {
      const errorDetail = `Habilitación de producto ya existente`;
      const errorObj = this.errMgr.get(moduleErrors.findEnableProduct.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newThirdProduct = { ...reqData, creation: new Date() };
    const actionResult = await enableProdCol.insertOne(newThirdProduct);
    this.logger.debug(`Habilitación de producto en tercero ${newThirdProduct} con resultado ${actionResult?.insertedId}`);
    return [null, { enabledProductId: actionResult?.insertedId }];
  }

  async disableThirdPartyProduct(reqData) {
    const { enabledProductId } = reqData;
    const enableProdCol = this.getDb().collection(PRODENABLE_COLLECTION);
    const enableProdDetail = await enableProdCol.findOne(this.productQueries.findById(enabledProductId));
    if (!enableProdDetail) {
      const errorDetail = `Habilitación de producto no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findEnableProduct.notFound, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await enableProdCol.deleteOne(this.productQueries.findById(enabledProductId));
    const result = actionResult?.result?.n > 0;
    this.logger.debug(`Eliminación del producto de tercerop ${enabledProductId} con resultado ${result}`);
    return [null, { result }];
  }

}

module.exports = ProductManager;
