const THIRDPARTY_COLLECTION = 'ThirdParties';
const GROUP_COLLECTION = 'ThirdPartyGroups';
const MEMBERS_COLLECTION = 'GroupMembers';
const ROLE_COLLECTION = 'EcosystemRoles';
const ROLERELATION_COLLECTION = 'RoleRelations';
const THIRD_RELATIONS = 'thirdPartyRelations';

const moduleErrors = {
  findThird: {
    notFound: ['01', 'No se encontraron terceros que se ajusten a la búsqueda'],
    roleNotFound: ['02', 'Rol para el tercero no encontrado'],
    missingData: ['03', 'Información insuficiente para localizar un tercero'],
  },
  findRole: {
    notFound: ['01', 'No se encontraron roles que se ajusten a la búsqueda'],
  },
  createThird: {
    alreadyExist: ['01', 'Tercero ya existe'],
  },
  modifyThird: {
    alreadyActive: ['01', 'Tercero ya está activo'],
    alreadyInactive: ['02', 'Tercero ya está inactivo'],
  },
  addRoleThirdParty: {
    roleExist: ['01', 'Tercero ya cuenta con el rol solicitado'],
    roleNotExist: ['01', 'Tercero no cuenta con el rol especificdo'],
  },
  findGroup: {
    notFound: ['01', 'No se encontraron grupos que se ajusten a la búsqueda'],
    alreadyActive: ['02', 'Grupo ya está activo'],
    alreadyInactive: ['03', 'Grupo ya está inactivo'],
  },
  createGroup: {
    alreadyExist: ['01', 'Tercero ya existe'],
  },
  findRelation: {
    notFound: ['01', 'Relación no existe'],
  },
  thirdPartyToGroup: {
    alreadyExist: ['01', 'Tercero ya vinculado al grupo'],
    notFound: ['02', 'Tercero no vinculado al grupo'],
  },
  relateThirdParties: {
    alreadyExist: ['01', 'Ya existe una relación con iguales características'],
    notFound: ['01', 'Relación entre terceros no encontrada'],
  },
};


class ThirdPartyManager {

  constructor(getDb, logger, errMgr, thirdpartyQueries) {
    this.getDb = getDb;
    this.logger = logger;
    this.errMgr = errMgr;
    this.thirdpartyQueries = thirdpartyQueries;
    this.errMgr.addModuleSet('thirdparties', moduleErrors);
  }

  async getRole(reqData) {
    const { roleId } = reqData;
    const rolesCol = this.getDb().collection(ROLE_COLLECTION);
    const roleDetail = await rolesCol.findOne(this.thirdpartyQueries.findById(roleId));
    if (!roleDetail) {
      const errorDetail = `Rol con Id ${roleId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findRole.notFound, errorDetail);
      return [errorObj, null];
    }
    return [null, roleDetail];
  }

  async getThirdParties(reqData = null, constraints = null) {
    const pageNumber = constraints.requestedPage;
    const recordsPerPage = constraints.recordsPerPage;
    // const currentFilter = constraints.currentFilter;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdParties = await thirdsCol.find()
      .skip(recordsPerPage * (pageNumber - 1))
      .limit(recordsPerPage)
      .toArray();
    return [null, thirdParties];
  }

  async getThirdParty(reqData) {
    const { thirdPartyId, idDocType, idDocNumber } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    let thirdDetail;
    if (thirdPartyId) {
      thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    } else if (!idDocType || !idDocNumber) {
      const errorObj = this.errMgr.get(moduleErrors.findThird.missingData);
      return [errorObj, null];
    } else {
      thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findThirdParty(idDocType, idDocNumber));
    }
    if (!thirdDetail) {
      const errorDetail = `Tercero no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
  }

  async createThirdParty(thirdPartyData) {
    const { idDocType, idDocNumber } = thirdPartyData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findThirdParty(idDocType, idDocNumber));
    if (thirdDetail) {
      const errorDetail = `Tercero con identificación ${idDocType} ${idDocNumber} ya existe`;
      const errorObj = this.errMgr.get(moduleErrors.createThird.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newThirdPartyData = { ...thirdPartyData, enabled: false, creation: new Date() };
    const actionResult = await thirdsCol.insertOne(newThirdPartyData);
    const thirdPartyId = actionResult?.insertedId;
    this.logger.debug(`Creación de un nuevo tercero ${newThirdPartyData} con inserción ${thirdPartyId}`);
    return [null, { thirdPartyId }];
  }

  async modifyThirdParty(reqData) {
    const { thirdPartyId } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    const thirdPartyData = { ...reqData };
    delete thirdPartyData.thirdPartyId;
    const actionResult = await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.updateObj(thirdPartyData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Modificación del tercero ${thirdPartyData} con resultado ${result}`);
    return [null, { result }];
  }

  async addExternalRefThirdParty(reqData) {
    const { thirdPartyId, externalSource, externalCode } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    const currentCoding = thirdDetail.externalCoding.find(item => item.externalSource === externalSource);
    if (currentCoding) {
      await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
        this.thirdpartyQueries.removeExternalRef(externalSource));
    }
    const externalRefData = { externalSource, externalCode };
    const actionResult = await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.addExternalRef(externalRefData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Adición de una referencia externa a un tercero ${externalRefData} con resultado ${result}`);
    return [null, { result }];
  }

  async ativateThirdParty(reqData) {
    const { thirdPartyId } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    if (thirdDetail.enabled) {
      const errorDetail = `Tercero con Id ${thirdPartyId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyThird.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.updateObj({ enabled: true }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación del tercero ${thirdPartyId} con resultado ${result}`);
    return [null, { result }];
  }

  async inativateThirdParty(reqData) {
    const { thirdPartyId } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!thirdDetail.enabled) {
      const errorDetail = `Tercero con Id ${thirdPartyId} actualmente inactivo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyThird.alreadyInactive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.updateObj({ enabled: false }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Inactivación del tercero ${thirdPartyId} con resultado ${result}`);
    return [null, { result }];
  }

  async deleteThirdParty(reqData) {
    const { thirdPartyId } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    if (thirdDetail.enabled) {
      const errorDetail = `Tercero con Id ${thirdPartyId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.modifyThird.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await thirdsCol.deleteOne(this.thirdpartyQueries.findById(thirdPartyId));
    const result = actionResult?.result?.n > 0;
    this.logger.debug(`Eliminación del tercero ${thirdPartyId} con resultado ${result}`);
    return [null, { result }];
  }

  async addRoleThirdParty(reqData) {
    const { thirdPartyId, roleId } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    const [getRoleErr, roleDetail] = await this.getRole({ roleId });
    if (getRoleErr || !roleDetail) {
      const errorDetail = `Rol con Id ${roleId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.roleNotFound, errorDetail);
      return [errorObj, null];
    }
    let existingThirdRole = (thirdDetail.roles && thirdDetail.roles.length > 0)
      ? thirdDetail.roles.find(item => item.roleId === roleId) : null;
    if (existingThirdRole) {
      const errorDetail = `Role con Id ${roleId} ya existe en el tercerop con Id ${thirdPartyId}`;
      const errorObj = this.errMgr.get(moduleErrors.addRoleThirdParty.roleExist, errorDetail);
      return [errorObj, null];
    }
    const roleData = { roleId, enabled: false, creation: new Date() };
    delete roleData.thirdPartyId;
    const actionResult = await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.addRole(roleData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Adición de un rol a un tercero ${roleData} con resultado ${result}`);
    return [null, { result }];
  }

  async activateRoleThirdParty(reqData) {
    const { thirdPartyId, roleId } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    let existingThirdRole = (thirdDetail.roles && thirdDetail.roles.length > 0)
      ? thirdDetail.roles.find(item => item.roleId === roleId) : null;
    if (!existingThirdRole) {
      const errorDetail = `Role con Id ${roleId} no existe en el tercerop con Id ${thirdPartyId}`;
      const errorObj = this.errMgr.get(moduleErrors.addRoleThirdParty.roleNotExist, errorDetail);
      return [errorObj, null];
    }
    const roleData = { roleId, enabled: true, creation: existingThirdRole.creation };
    delete roleData.thirdPartyId;
    await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.removeRole(roleId));
    const actionResult = await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.addRole(roleData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación de un rol a un tercero ${roleData} con resultado ${result}`);
    return [null, { result }];
  }

  async inactivateRoleThirdParty(reqData) {
    const { thirdPartyId, roleId } = reqData;
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    let existingThirdRole = (thirdDetail.roles && thirdDetail.roles.length > 0)
      ? thirdDetail.roles.find(item => item.roleId === roleId) : null;
    if (!existingThirdRole) {
      const errorDetail = `Role con Id ${roleId} no existe en el tercerop con Id ${thirdPartyId}`;
      const errorObj = this.errMgr.get(moduleErrors.addRoleThirdParty.roleNotExist, errorDetail);
      return [errorObj, null];
    }
    const roleData = { roleId, enabled: false, creation: existingThirdRole.creation };
    delete roleData.thirdPartyId;
    await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.removeRole(roleId));
    const actionResult = await thirdsCol.updateOne(this.thirdpartyQueries.findById(thirdPartyId),
      this.thirdpartyQueries.addRole(roleData));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Inactivación de un rol a un tercero ${roleData} con resultado ${result}`);
    return [null, { result }];
  }

  /**
   * Grupos de terceros
   */

  async createGroup(groupData) {
    const { ownerId, groupName } = groupData;
    const groupsCol = this.getDb().collection(GROUP_COLLECTION);
    const groupDetail = await groupsCol.findOne(this.thirdpartyQueries.findGroup(ownerId, groupName));
    if (groupDetail) {
      const errorDetail = `Grupo ${groupName} ya existe para el tercero ${ownerId}`;
      const errorObj = this.errMgr.get(moduleErrors.createGroup.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const newGroupData = { ...groupData, enabled: false, creation: new Date() };
    const actionResult = await groupsCol.insertOne(newGroupData);
    this.logger.debug(`Creación de un nuevo grupo ${groupData} con resultado ${actionResult?.insertedId}`);
    return [null, { groupId: actionResult?.insertedId }];
  }

  async activateGroup(reqData) {
    const { groupId } = reqData;
    const groupsCol = this.getDb().collection(GROUP_COLLECTION);
    const groupDetail = await groupsCol.findOne(this.thirdpartyQueries.findById(groupId));
    if (!groupDetail) {
      const errorDetail = `Grupo con Id ${groupId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.notFound, errorDetail);
      return [errorObj, null];
    }
    if (groupDetail.enabled) {
      const errorDetail = `Grupo con Id ${groupId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await groupsCol.updateOne(this.thirdpartyQueries.findById(groupId),
      this.thirdpartyQueries.updateObj({ enabled: true }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación del grupo ${groupId} con reultado ${result}`);
    return [null, { result }];
  }

  async inactivateGroup(reqData) {
    const { groupId } = reqData;
    const groupsCol = this.getDb().collection(GROUP_COLLECTION);
    const groupDetail = await groupsCol.findOne(this.thirdpartyQueries.findById(groupId));
    if (!groupDetail) {
      const errorDetail = `Grupo con Id ${groupId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!groupDetail.enabled) {
      const errorDetail = `Grupo con Id ${groupId} actualmente inactivo`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.alreadyInactive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await groupsCol.updateOne(this.thirdpartyQueries.findById(groupId),
      this.thirdpartyQueries.updateObj({ enabled: false }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Activación del grupo ${groupId} con reultado ${result}`);
    return [null, { result }];
  }

  async deleteGroup(reqData) {
    const { groupId } = reqData;
    const groupsCol = this.getDb().collection(GROUP_COLLECTION);
    const groupDetail = await groupsCol.findOne(this.thirdpartyQueries.findById(groupId));
    if (!groupDetail) {
      const errorDetail = `Grupo con Id ${groupId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.notFound, errorDetail);
      return [errorObj, null];
    }
    if (groupDetail.enabled) {
      const errorDetail = `Grupo con Id ${groupId} actualmente activo`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.alreadyActive, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await groupsCol.deleteOne(this.thirdpartyQueries.findById(groupId));
    const result = actionResult?.result?.n > 0;
    this.logger.debug(`Eliminación del grupo ${groupId} con resultado ${result}`);
    return [null, { result }];
  }

  async existThirdGroup(thirdPartyId, groupId) {
    const membersCol = this.getDb().collection(MEMBERS_COLLECTION);
    const memberDetail = await membersCol.findOne(this.thirdpartyQueries.findThirdGroup(thirdPartyId, groupId));
    return memberDetail !== null && memberDetail !== undefined;
  }

  async addThirdPartyToGroup(reqData) {
    const { groupId, thirdPartyId } = reqData;
    const groupsCol = this.getDb().collection(GROUP_COLLECTION);
    const groupDetail = await groupsCol.findOne(this.thirdpartyQueries.findById(groupId));
    if (!groupDetail) {
      const errorDetail = `Grupo con Id ${groupId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.notFound, errorDetail);
      return [errorObj, null];
    }
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const thirdDetail = await thirdsCol.findOne(this.thirdpartyQueries.findById(thirdPartyId));
    if (!thirdDetail) {
      const errorDetail = `Tercero con Id ${thirdPartyId} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    const existingThirdGroup = await existThirdGroup(thirdPartyId, groupId);
    if (existingThirdGroup) {
      const errorDetail = `Tercero con Id ${thirdPartyId} ya existe en el grupo con Id ${groupId}`;
      const errorObj = this.errMgr.get(moduleErrors.thirdPartyToGroup.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const membersCol = this.getDb().collection(MEMBERS_COLLECTION);
    const newMemberData = { ...reqData, enabled: false, creation: new Date() };
    const actionResult = await membersCol.insertOne(newMemberData);
    this.logger.debug(`Adición de un tercero a un grupo ${newMemberData} con resultado ${actionResult?.insertedId}`);
    return [null, { membershipId: actionResult?.insertedId }];
  }

  async deleteThirdPartyFromGroup(reqData) {
    const { membershipId } = reqData;
    const membersCol = this.getDb().collection(MEMBERS_COLLECTION);
    const membershipDetail = await membersCol.findOne(this.thirdpartyQueries.findById(membershipId));
    if (!membershipDetail) {
      const errorDetail = `No existe membresía a grupo con Id ${membershipId}`;
      const errorObj = this.errMgr.get(moduleErrors.thirdPartyToGroup.notFound, errorDetail);
      return [errorObj, null];
    }
    const actionResult = await membersCol.deleteOne(this.thirdpartyQueries.findById(membershipId));
    const result = actionResult?.result?.n > 0;
    this.logger.debug(`Eliminación de membreía ${membershipId} con resultado ${result}`);
    return [null, { result }];
  }

  async moveGroupMembers(reqData) {
    const { sourceGroupId, destinationGroupId } = reqData;
    const groupsCol = this.getDb().collection(GROUP_COLLECTION);
    const sourceGroupDetailPrms = groupsCol.findOne(this.thirdpartyQueries.findById(sourceGroupId));
    const destGroupDetailPrms = groupsCol.findOne(this.thirdpartyQueries.findById(destinationGroupId));
    const [sourceGroupDetail, destGroupDetail] = await Promise.all([sourceGroupDetailPrms, destGroupDetailPrms]);
    if (!sourceGroupDetail) {
      const errorDetail = `Grupo con Id ${sourceGroupDetail} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!destGroupDetail) {
      const errorDetail = `Grupo con Id ${destGroupDetail} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findGroup.notFound, errorDetail);
      return [errorObj, null];
    }
    const membersCol = this.getDb().collection(MEMBERS_COLLECTION);
    const actionResult = await membersCol.update(this.thirdpartyQueries.findMembers(sourceGroupId),
      this.thirdpartyQueries.updateObj({ groupId: destinationGroupId }));
    const result = actionResult?.result?.nModified > 0;
    this.logger.debug(`Movimiento de miembros de grupo ${sourceGroupId} hacia ${destinationGroupId} con resultado ${result}`);
    return [null, { result }];
  }

  /**
   * Relacionamiento de terceros
   */

  async relateThirdParties(reqData) {
    const { primaryThirdParty, secondaryThirdParty, roleRelationId } = reqData;
    const thirdRelationsCol = this.getDb().collection(THIRD_RELATIONS);
    const thirdRelationDetail = await thirdRelationsCol
      .findOne(this.thirdpartyQueries.findThirdRelation(primaryThirdParty, secondaryThirdParty, roleRelationId));
    if (thirdRelationDetail) {
      const errorDetail = `Relación existente entre ${firstThirdDetail} y ${secondThirdDetail} de tipo ${roleRelationId}`;
      const errorObj = this.errMgr.get(moduleErrors.relateThirdParties.alreadyExist, errorDetail);
      return [errorObj, null];
    }
    const thirdsCol = this.getDb().collection(THIRDPARTY_COLLECTION);
    const roleRelationsCol = this.getDb().collection(ROLERELATION_COLLECTION);
    const relationDetailPrms = roleRelationsCol.findOne(this.thirdpartyQueries.findById(roleRelationId));
    const firstThirdDetailPrms = thirdsCol.findOne(this.thirdpartyQueries.findById(primaryThirdParty));
    const secondThirdDetailPrms = thirdsCol.findOne(this.thirdpartyQueries.findById(secondaryThirdParty));
    const [firstThirdDetail, secondThirdDetail, relationDetail] = Promise
      .all([firstThirdDetailPrms, secondThirdDetailPrms, relationDetailPrms]);
    if (!relationDetail || !relationDetail.primaryRoleId || !relationDetail.secondaryRoleId) {
      const errorDetail = `Relación entre roles con Id ${roleRelationId} no encontrada`;
      const errorObj = this.errMgr.get(moduleErrors.findRelation.notFound, errorDetail);
      return [errorObj, null];
    }
    if (!firstThirdDetail) {
      const errorDetail = `Tercero con Id ${firstThirdDetail} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    const firstHasRole = firstThirdDetail.roles && firstThirdDetail.roles
      .find(item => item.roleId === relationDetail.primaryRoleId);
    if (!firstHasRole) {
      const errorDetail = `Tercero con Id ${firstThirdDetail} no tiene el rol ${relationDetail.primaryRoleId}`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.roleNotFound, errorDetail);
      return [errorObj, null];
    }
    if (!secondThirdDetail) {
      const errorDetail = `Tercero con Id ${secondThirdDetail} no encontrado`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.notFound, errorDetail);
      return [errorObj, null];
    }
    const secondHasRole = secondThirdDetail.roles && firstThirdDetail.roles
      .find(item => item.roleId === relationDetail.secondaryRoleId);
    if (!secondHasRole) {
      const errorDetail = `Tercero con Id ${secondThirdDetail} no tiene el rol ${relationDetail.secondaryRoleId}`;
      const errorObj = this.errMgr.get(moduleErrors.findThird.roleNotFound, errorDetail);
      return [errorObj, null];
    }
    const newRelationData = { primaryThirdParty, secondaryThirdParty, roleRelationId, enabled: true, creation: new Date() };
    const actionResult = await thirdRelationsCol.insertOne(newRelationData);
    this.logger.debug(`Creación de relación ${newRelationData} con resultado ${actionResult?.insertedId}`);
    return [null, { thirdPartyRelationId: actionResult?.insertedId }];
  }

  async deleteThirdPartiesRelation(reqData) {
    const { thirdPartyRelationId } = reqData;
    const thirdRelationsCol = this.getDb().collection(THIRD_RELATIONS);
    const thirdRelationDetail = await thirdRelationsCol.findOne(this.thirdpartyQueries.findById(thirdPartyRelationId));
    if (!thirdRelationDetail) {
      const errorObj = this.errMgr.get(moduleErrors.relateThirdParties.notFound);
      return [errorObj, null];
    }
    const actionResult = await thirdRelationsCol.deleteOne(this.thirdpartyQueries.findById(thirdPartyRelationId));
    const result = actionResult?.result?.n > 0;
    this.logger.debug(`Eliminación de la relación ${thirdPartyRelationId} con resultado ${result}`);
    return [null, { result }];
  }

  async unrelateThirdParties(reqData) {
    const { primaryThirdParty, secondaryThirdParty, roleRelationId } = reqData;
    const thirdRelationsCol = this.getDb().collection(THIRD_RELATIONS);
    const thirdRelationDetail = await thirdRelationsCol.findOne(this.thirdpartyQueries.findThirdRelation(primaryThirdParty, secondaryThirdParty, roleRelationId));
    if (!thirdRelationDetail) {
      const errorObj = this.errMgr.get(moduleErrors.relateThirdParties.alreadyExist);
      return [errorObj, null];
    }
    const thirdPartyRelationId = thirdRelationDetail._id;
    const actionResult = await thirdRelationsCol.deleteOne(this.thirdpartyQueries.findById(thirdPartyRelationId));
    const result = actionResult?.result?.n > 0;
    this.logger.debug(`Eliminación de la relación ${thirdPartyRelationId} entre ${primaryThirdParty}, ${secondaryThirdParty} de tipo ${roleRelationId} resultado ${result}`);
    return [null, { result }];
  }
}

module.exports = ThirdPartyManager;
